import pytest
from .. import apache
import asyncio
from asyncio import coroutine
import os
import re


def test_heartbeat_lines_format():
	gen = apache(out_path='./test_heartbeat_lines_format.txt', lines=['heartbeat'], forever=False, count=1)
	gen.run()
	
	try:
		f = open('./test_heartbeat_lines_format.txt')
		line = f.readlines()[0]
		#fields = line.split()
		#assert len(fields) == 8
		# Extract the time field
		log_time = re.findall(r'\[(.*?)\]', line)
		assert len(log_time) == 1
		# Extract the message field
		log_msg = re.findall(r'\"(.*?)\"', line)
		assert len(log_msg) == 1
		assert log_msg[0] == 'HEARTBEAT'
	except:
		assert False	

	
def	test_access_lines_format():
	gen = apache(out_path='./test_access_lines_format.txt', lines=['access'], forever=False, count=1)
	gen.run()

	try:
		f = open('./test_access_lines_format.txt')
		line = f.readlines()[0]
		# Extract the time field
		log_time = re.findall(r'\[(.*?)\]', line)
		assert len(log_time) == 1
		# Extract the message field
		log_msg = re.findall(r'\"(.*?)\"', line)
		assert len(log_msg) == 1

	except:
		assert False


# Test param: lines
def test_lines_control():
	gen = apache(out_path='./test_lines_control.txt', lines=['heartbeat', 'access'], methods=['GET', 'PUT', 'POST', 'DELETE'], forever=False, count=10)
	gen.run()

	lines_li = set()

	try:
		f = open('./test_lines_control.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			if log_msg == 'HEARTBEAT':
				lines_li.add('heartbeat')
			else:
				log_method = log_msg.split()[0]
				if log_method in ['GET', 'PUT', 'POST', 'DELETE']:
					lines_li.add('access')

		assert lines_li == set(['heartbeat', 'access'])

	except:
		assert False


# Test param: methods
def test_access_lines_method():
	# Test GET generation
	gen = apache(out_path='./test_access_lines_method.txt', lines=['access'], methods=['GET'], forever=False, count=3)
	gen.run()

	try:
		f = open('./test_access_lines_method.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			# Extract the http method
			log_method = log_msg.split()[0]
			assert log_method == 'GET'

	except:
		assert False

	# Test PUT generation
	gen = apache(out_path='./test_access_lines_method.txt', lines=['access'], methods=['PUT'], forever=False, count=3)
	gen.run()

	try:
		f = open('./test_access_lines_method.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			# Extract the http method
			log_method = log_msg.split()[0]
			assert log_method == 'PUT'

	except:
		assert False

	# Test POST generation
	gen = apache(out_path='./test_access_lines_method.txt', lines=['access'], methods=['POST'], forever=False, count=3)
	gen.run()

	try:
		f = open('./test_access_lines_method.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			# Extract the http method
			log_method = log_msg.split()[0]
			assert log_method == 'POST'
	except:
		assert False

	# Test DELETE generation
	gen = apache(out_path='./test_access_lines_method.txt', lines=['access'], methods=['DELETE'], forever=False, count=3)
	gen.run()

	try:
		f = open('./test_access_lines_method.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			# Extract the http method
			log_method = log_msg.split()[0]
			assert log_method == 'DELETE'
	except:
		assert False

# Test param: methods_p
def test_access_lines_method_dist():
	gen = apache(out_path='./test_access_lines_method_dist.txt', lines=['access'], methods=['GET', 'POST', 'PUT', 'DELETE'], methods_p=[1.0, 0, 0, 0], forever=False, count=3)
	gen.run()

	try:
		f = open('./test_access_lines_method_dist.txt')
		lines = f.readlines()
		for line in lines:
			# Extract the message field
			log_msg = re.findall(r'\"(.*?)\"', line)[0]
			# Extract the http method
			log_method = log_msg.split()[0]
			assert log_method == 'GET'
	except:
		assert False







import os
import random
import json
import logging
import argparse
import asyncio
import datetime
from asyncio import coroutine
import numpy


# Apache Access Logs
class apache(object):

	def __init__(self, out_path='./apache.log', lines=['heartbeat', 'access'], heartbeat_interval=0.1, access_interval=[0.1, 2], methods=['GET', 'POST', 'PUT', 'DELETE'], methods_p = [0.7, 0.1, 0.1, 0.1], mode='normal', forever=True, count=1):
		# Assign the lines to generate
		self.lines = lines
		self.lines_full = ['heartbeat', 'access']
		self.lines_gen = [self.heartbeat_lines(), self.access_lines()]
		# Assign the http methods to generate
		self.methods = methods	
		# Assign the methods distribution
		self.methods_p = methods_p
		# Run forever or not
		self.forever = forever
		# Total # of logs to generate
		self.count = count
		# Assign the intervals
		self.heartbeat_interval = heartbeat_interval
		self.access_interval = access_interval
		# Assign the generator mode
		self.mode = mode

		# Instantiate the logger
		self.log = logging.getLogger('Gen')
		# Set the level
		logging.basicConfig(level=logging.INFO)
		# Instantiate a file Handler
		out = logging.FileHandler(out_path, mode='w')

		log_format = logging.Formatter("%(message)s")
		# Set the Formatter for this Handler to form
		out.setFormatter(log_format)
		# Add the file Handler 'out' to the logger'log'
		self.log.addHandler(out)



	def run(self):
		loop = asyncio.new_event_loop()
		asyncio.set_event_loop(loop)
		try:
			loop.run_until_complete(
				asyncio.wait([self.lines_gen[self.lines_full.index(x)] for x in self.lines])
		)
		finally:
			loop.close()



	@coroutine
	def heartbeat_lines(self):
		while self.forever or self.count > 0:
			t = datetime.datetime.now().strftime('%d/%b/%Y:%H:%M:%S -0700')	
			self.log.info('- - - [%s] "%s" - -', t, 'HEARTBEAT')
			if self.count > 0:
				self.count -= 1
			yield from asyncio.sleep(self.heartbeat_interval)


	@coroutine
	def access_lines(self):
		while self.forever or self.count > 0:
			ip = '.'.join(str(random.randint(0, 255)) for i in range(4))
			user_identifier = '-'
			user_id = 'frank'
			t = datetime.datetime.now().strftime('%d/%b/%Y:%H:%M:%S -0700')

			method = numpy.random.choice(self.methods, p=self.methods_p)
			#method = random.choice(self.methods)
			#resource = self.resources[random.randint(0, len(self.resources)-1)]
			resource = '/apache_pb.gif'
			#version = self.versions[random.randint(0, len(self.versions)-1)]
			version = 'HTTP/1.0'
			msg = method + " " + resource + " " + version
			#code = numpy.random.choice(self.codes, p=self.codes_dist)
			code = '200'
			size = random.randint(1024, 10240)
			self.log.info('%s %s %s [%s] "%s" %s %s', ip, user_identifier, user_id, t, msg, code, size)
			if self.count > 0:
				self.count -= 1

			# According to the generator mode:
			# 'normal' mode - uniform distribution between min & max intervals
			if self.mode == 'normal':
				yield from asyncio.sleep(random.uniform(self.access_interval[0], self.access_interval[1]))
			# 'push' mode - at highest rate
			elif self.mode == 'push':
				yield from asyncio.sleep(self.access_interval[0])
			# 'spike' mode
			elif self.mode == 'spike':
				mean = (self.access_interval[0]+self.access_interval[1])/2
				# Standard deviation
				sigma = (self.access_interval[1]-self.access_interval[0])/2
				yield from asyncio.sleep(numpy.random.normal(mean, sigma))
			else:
				yield from asyncio.sleep(random.uniform(self.access_interval[0], self.access_interval[1]))







from setuptools import setup, find_packages

# Parse the requirements.txt to get the dependencies
#with open('requirements.txt') as f:
#	required = f.read().splitlines()

setup(
	name = 'lunaticlog',
	version = '0.1.0',
	description = 'A log generator to test your monitor system.',
	url = 'https://github.com/xuwenyihust/lunaticlog',
	author = 'Wenyi Xu',
	author_email = 'wenyixu101@gmail.com',
	license = 'MIT',
	keywords='log monitor apache',

	packages = find_packages(),

#	install_requires = required,

	package_data={
   		'configuration': ['conf/*.json'],
	},

	test_suite = 'tests',
	setup_requires=['pytest-runner'],
	tests_require = ['pytest']
)
